﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using DA = HeuristicLab.Services.Access.DataAccess;
using DT = HeuristicLab.Services.Access.DataTransfer;

namespace HeuristicLab.Services.Access {
  [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall)]
  public class AccessService : IAccessService {

    #region Client Members
    public bool ClientExists(Guid id) {
      if (id != Guid.Empty) {
        using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
          return (context.Resources.Where(x => x.Id == id).Count() != 0);
        }
      }
      return false;
    }

    public DT.Client GetClient(Guid id) {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        var query = from c in context.GetTable<DA.Resource>().OfType<DA.Client>()
                    where c.Id == id
                    select c;

        return Convert.ToDto(query.FirstOrDefault());
      }
    }

    public IEnumerable<DT.Client> GetClients(IEnumerable<Guid> ids) {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        var query = from c in context.GetTable<DA.Resource>().OfType<DA.Client>()
                    where ids.Contains(c.Id)
                    select Convert.ToDto(c);
        return query.ToList();
      }
    }

    public IEnumerable<DT.Client> GetAllClients() {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        var query = from c in context.GetTable<DA.Resource>().OfType<DA.Client>()
                    select Convert.ToDto(c);
        return query.ToList();
      }
    }

    public Guid AddClient(DT.Client client) {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        DA.Client entity = Convert.ToEntity(client);
        context.Resources.InsertOnSubmit(entity);
        context.SubmitChanges();
        return entity.Id;
      }
    }

    public void UpdateClient(DT.Client client) {
      AddClient(client);
    }

    public void DeleteClient(DT.Client client) {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        context.Resources.DeleteOnSubmit(Convert.ToEntity(client));
        context.SubmitChanges();
      }
    }
    #endregion

    #region ClientGroup
    public IEnumerable<DT.ClientGroup> GetAllClientGroups() {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        var query = from c in context.GetTable<DA.Resource>().OfType<DA.ClientGroup>()
                    select Convert.ToDto(c);
        return query.ToList();
      }
    }

    public IEnumerable<DT.ClientGroup> GetClientGroups(IEnumerable<Guid> ids) {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        var query = from c in context.GetTable<DA.Resource>().OfType<DA.ClientGroup>()
                    where ids.Contains(c.Id)
                    select Convert.ToDto(c);
        return query.ToList();
      }
    }

    public Guid AddClientGroup(DT.ClientGroup group) {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        var entity = Convert.ToEntity(group);
        context.Resources.InsertOnSubmit(entity);
        context.SubmitChanges();
        return entity.Id;
      }
    }

    public void UpdateClientGroup(DT.ClientGroup group) {
      AddClientGroup(group);
    }

    public void DeleteClientGroup(DT.ClientGroup group) {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        context.Resources.DeleteOnSubmit(Convert.ToEntity(group));
        context.SubmitChanges();
      }
    }

    public void AddResourceToGroup(DT.Resource resource, DT.UserGroup group) {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        DA.ResourceResourceGroup rrg = new DA.ResourceResourceGroup() {
          ResourceId = resource.Id,
          ResourceGroupId = group.Id
        };

        context.ResourceResourceGroups.InsertOnSubmit(rrg);
        context.SubmitChanges();
      }
    }

    public void RemoveResourceFromGroup(DT.Resource resource, DT.UserGroup group) {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        var query = context.ResourceResourceGroups.Where(x => x.ResourceId == resource.Id && x.ResourceGroupId == group.Id);
        if (query.Count() > 0) {
          context.ResourceResourceGroups.DeleteOnSubmit(query.First());
          context.SubmitChanges();
        }
      }
    }
    #endregion

    #region ClientGroupMapping
    public IEnumerable<DT.ClientGroupMapping> GetClientGroupMapping() {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        var query = from c in context.GetTable<DA.ResourceResourceGroup>()
                    select Convert.ToDto(c);
        return query.ToList();
      }
    }
    #endregion

    #region Resource
    public IEnumerable<DT.Resource> GetResources() {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        var query = from r in context.Resources
                    select Convert.ToDto(r);
        return query.ToList();
      }
    }
    #endregion

    #region ClientLog
    public DT.ClientLog GetLastClientLog(Guid clientId) {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        var query = from r in context.ClientLogs
                    where r.ResourceId == clientId
                    select r;
        return Convert.ToDto(query.OrderBy(x => x.Timestamp).LastOrDefault());
      }
    }

    public IEnumerable<DT.ClientLog> GetClientLogs(Guid clientId) {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        var query = from r in context.ClientLogs
                    where r.ResourceId == clientId
                    select Convert.ToDto(r);
        return query.ToList();
      }
    }

    public IEnumerable<DT.ClientLog> GetClientLogsSince(DateTime startDate) {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        var query = from r in context.ClientLogs
                    where r.Timestamp >= startDate
                    select Convert.ToDto(r);
        return query.ToList();
      }
    }

    public void AddClientLog(DT.ClientLog log) {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        context.ClientLogs.InsertOnSubmit(Convert.ToEntity(log));
        context.SubmitChanges();
      }
    }

    public void DeleteClientLog(DT.ClientLog log) {
      using (DA.ClientManagementDataContext context = new DA.ClientManagementDataContext()) {
        context.ClientLogs.DeleteOnSubmit(Convert.ToEntity(log));
        context.SubmitChanges();
      }
    }
    #endregion

    #region User
    public DT.User Login() {
      throw new System.NotImplementedException();
    }

    public IEnumerable<DT.User> GetAllUsers() {
      throw new System.NotImplementedException();
    }

    public IEnumerable<DT.User> GetUsers(IEnumerable<DT.User> ids) {
      throw new System.NotImplementedException();
    }

    public DT.User AddUser(DT.User user) {
      throw new System.NotImplementedException();
    }

    public void DeleteUser(DT.User user) {
      throw new System.NotImplementedException();
    }

    public void UpdateUser(DT.User user) {
      throw new System.NotImplementedException();
    }

    public void AddUserToRole(DT.Role role, DT.User user) {
      throw new System.NotImplementedException();
    }

    public void RemoveUserFromRole(DT.Role role, DT.User user) {
      throw new System.NotImplementedException();
    }

    public void ResetPassword(DT.User user, string password) {
      throw new System.NotImplementedException();
    }
    #endregion

    public IEnumerable<DT.UserGroup> GetAllUserGroups() {
      throw new System.NotImplementedException();
    }

    public IEnumerable<DT.UserGroup> GetUserGroups(IEnumerable<Guid> ids) {
      throw new System.NotImplementedException();
    }

    public DT.UserGroup AddUserGroup(DT.UserGroup group) {
      throw new System.NotImplementedException();
    }

    public void UpdateUserGroup(DT.UserGroup group) {
      throw new System.NotImplementedException();
    }

    public void DeleteUserGroup(DT.UserGroup group) {
      throw new System.NotImplementedException();
    }

    public void AddUserGroupBaseToGroup(DT.UserGroupBase resource, DT.UserGroup group) {
      throw new System.NotImplementedException();
    }

    public void RemoveUserGroupBaseFromGroup(DT.UserGroupBase resource, DT.UserGroup group) {
      throw new System.NotImplementedException();
    }

    public IEnumerable<DT.UserGroupBase> GetUsersAndGroups() {
      throw new System.NotImplementedException();
    }

    public IEnumerable<DT.UserGroupMapping> GetUserGroupMapping() {
      throw new System.NotImplementedException();
    }

    public IEnumerable<DT.Role> GetRoles() {
      throw new System.NotImplementedException();
    }

    public DT.Role AddRole(DT.Role role) {
      throw new System.NotImplementedException();
    }

    public void UpdateRole(DT.Role role) {
      throw new System.NotImplementedException();
    }

    public void DeleteRole(DT.Role role) {
      throw new System.NotImplementedException();
    }

    public IEnumerable<DT.Role> GetUserRoles(DT.User user) {
      throw new System.NotImplementedException();
    }

    public void ReportError(DT.ClientError error) {
      throw new System.NotImplementedException();
    }

    public IEnumerable<DT.ClientError> GetClientErrors() {
      throw new System.NotImplementedException();
    }

    public IEnumerable<DT.ClientError> GetLastClientErrors(DateTime startDate) {
      throw new System.NotImplementedException();
    }

    public void DeleteError(DT.ClientError error) {
      throw new System.NotImplementedException();
    }
  }
}
